# Third party
from django.utils.translation import gettext as _

# Local application / specific library imports
from ..checks import custom_list


def importance():
    """Scripts with higher importance will be executed in first.

    Returns:
        int -- Importance of the script.
    """
    return 1


def run(site):
    """Checks the number of words in the extracted content.

    Arguments:
        site {Site} -- Structure containing a good amount of resources from the targeted webpage.
    """

    short_content = custom_list.CustomList(
        name=_("Content is too short"),
        settings=_("at least {min} words, more than {min2} if possible").format(
            min=site.settings.SEO_SETTINGS["content_words_number"][0],
            min2=site.settings.SEO_SETTINGS["content_words_number"][1],
        ),
        description=_(
            "Longer articles tend to be better ranked, but will require better writing skills than shorter articles."
        ),
    )

    nb_words = len(site.content_text.split())
    short_content.found = nb_words
    short_content.searched_in = [site.content_text]

    # too few words
    if nb_words < site.settings.SEO_SETTINGS["content_words_number"][0]:
        site.problems.append(short_content)

    elif nb_words < site.settings.SEO_SETTINGS["content_words_number"][1]:
        site.warnings.append(short_content)

    else:
        short_content.name = _("Content length is right")
        site.success.append(short_content)



# Standard Library
import re

# Third party
from django.utils.translation import gettext as _, pgettext

# Local application / specific library imports
from ..checks import custom_list


def importance():
    """Scripts with higher importance will be executed in first.

    Returns:
        int -- Importance of the script.
    """
    return 1


def run(site):
    """Verifies that only one h1 tag is present, and that it contains at least one keyword.

    Arguments:
        site {Site} -- Structure containing a good amount of resources from the targeted webpage.
    """
    too_much_h1 = custom_list.CustomList(
        name=_("Too much h1 tags"),
        settings=_("exactly 1"),
        description=_(
            "Google is not really concerned about the number of h1 tags on your page, but Bing clearly indicates in its guidelines for webmasters to use only one h1 tag per page."
        ),
    )

    right_number_h1 = custom_list.CustomList(
        name=_("H1 tag found"),
        settings=_("exactly 1"),
        description=too_much_h1.description,
    )

    not_enough_h1 = custom_list.CustomList(
        name=_("No h1 tag"),
        settings=_("exactly 1"),
        description=too_much_h1.description,
    )

    no_keywords = custom_list.CustomList(
        name=_("No keyword in h1"),
        settings=_("at least one"),
        description=_(
            "The h1 tag represent the main title of your page, and you may populate it with appropriate content in order to ensure that users (and search engines!) will understand correctly your page."
        ),
    )

    enough_keywords = custom_list.CustomList(
        name=_("Keyword found in h1"),
        settings=_("at least one"),
        description=no_keywords.description,
    )

    h1 = site.soup.find_all("h1")

    if len(h1) > 1:
        too_much_h1.found = len(h1)
        too_much_h1.searched_in = [t.text for t in h1]
        site.problems.append(too_much_h1)

    elif not h1:
        not_enough_h1.found = pgettext("masculin", "none")
        site.problems.append(not_enough_h1)

    else:

        right_number_h1.found = len(h1)
        right_number_h1.searched_in = [t.text for t in h1]
        site.success.append(right_number_h1)

        # h1 text can be content of alt tag in img
        if not h1[0].text and h1[0].find("img", {"alt": True}):
            h1_text = h1[0].find("img")["alt"].lower()
        # of it can be the text in h1
        else:
            h1_text = h1[0].text.lower()

        occurence = []
        for keyword in site.keywords:
            occurence.append(
                sum(
                    1
                    for _ in re.finditer(
                        r"\b%s\b" % re.escape(keyword.lower()), h1_text,
                    )
                )
            )
        # if no keyword is found in h1
        if not any(i > 0 for i in occurence):
            no_keywords.found = pgettext("masculin", "none")
            no_keywords.searched_in = [t.text for t in h1]
            site.problems.append(no_keywords)
        else:
            enough_keywords.found = max(i for i in occurence)
            enough_keywords.searched_in = [t.text for t in h1]
            site.success.append(enough_keywords)



# Standard Library
import re

# Third party
from django.utils.translation import gettext as _, ngettext, pgettext

# Local application / specific library imports
from ..checks import custom_list


def importance():
    """Scripts with higher importance will be executed in first.

    Returns:
        int -- Importance of the script.
    """
    return 1


def run(site):
    """Checks that only one meta description tag is present in the page, that it is the right length and that it contains at least one keyword.

    Arguments:
        site {Site} -- Structure containing a good amount of resources from the targeted webpage.
    """
    length_short = custom_list.CustomList(
        name=_("Meta description is too short"),
        settings=_("between {rule_low} and {rule_high} chars ").format(
            rule_low=site.settings.SEO_SETTINGS["meta_description_length"][0],
            rule_high=site.settings.SEO_SETTINGS["meta_description_length"][1],
        ),
        description=_(
            "The meta description tag can be displayed in search results if it has the right length, and can influence users. Knowing that Google classifies sites according to user behaviour, it is important to have a relevant description."
        ),
    )

    length_long = custom_list.CustomList(
        name=_("Meta description is too long"),
        settings=length_short.settings,
        description=length_short.description,
    )

    length_success = custom_list.CustomList(
        name=_("Meta description length is correct"),
        settings=length_short.settings,
        description=length_short.description,
    )

    keywords_bad = custom_list.CustomList(
        name=_("No keyword in meta description"),
        settings=pgettext("masculin", "at least one"),
        description=length_short.description,
    )

    keywords_good = custom_list.CustomList(
        name=_("Keywords were found in meta description"),
        settings=pgettext("masculin", "at least one"),
        description=length_short.description,
    )

    too_much_meta = custom_list.CustomList(
        name=_("Too much meta description tags"),
        settings=pgettext("feminin", "only one"),
        description=_(
            "Although some people write one meta description by targeted keyword, this is still an uncommon practice that is not yet recognized by all search engines."
        ),
    )

    meta_description_only_one = custom_list.CustomList(
        name=_("Only one meta description tag"),
        settings=pgettext("feminin", "only one"),
        found=pgettext("feminin", "one"),
        description=too_much_meta.description,
    )

    no_meta_description = custom_list.CustomList(
        name=_("No meta description"),
        settings=pgettext("feminin", "needed"),
        found=pgettext("description", "none"),
        description=length_short.description,
    )

    meta_description_present = custom_list.CustomList(
        name=_("Meta description is present"),
        settings=pgettext("feminin", "needed"),
        found=pgettext("feminin", "one"),
        description=_(
            "The meta description tag can be displayed in search results if it has the right length, and can influence users. Knowing that Google classifies sites according to user behaviour, it is important to have a relevant description."
        ),
    )

    meta = site.soup.find_all("meta")
    found_meta_description = False
    number_meta_description = 0
    meta_description = []

    for tag in meta:
        if (
            "name" in tag.attrs
            and tag.attrs["name"] == "description"
            and "content" in tag.attrs
            and tag.attrs["content"] != ""
        ):
            number_meta_description += 1
            found_meta_description = True

            meta_description.append(tag.attrs["content"])
            length = len(tag.attrs["content"])

            # too short
            if length < site.settings.SEO_SETTINGS["meta_description_length"][0]:

                length_short.found = ngettext(
                    "%(words)d char", "%(words)d chars", length
                ) % {"words": length}
                length_short.searched_in = meta_description
                site.problems.append(length_short)

            # too long
            elif length > site.settings.SEO_SETTINGS["meta_description_length"][1]:

                length_long.found = str(length)
                length_long.searched_in = meta_description
                site.problems.append(length_long)

            # perfect
            else:

                length_success.found = str(length)
                length_success.searched_in = meta_description
                site.success.append(length_success)

            occurence = []
            for keyword in site.keywords:
                occurence.append(
                    sum(
                        1
                        for _ in re.finditer(
                            r"\b%s\b" % re.escape(keyword.lower()),
                            tag.attrs["content"].lower(),
                        )
                    )
                )
            # if no keyword is found in description
            if not any(i > 0 for i in occurence):

                keywords_bad.found = 0
                keywords_bad.searched_in = meta_description
                site.warnings.append(keywords_bad)

            # perfect
            else:
                keywords_good.found = max(i for i in occurence)
                keywords_good.searched_in = meta_description
                site.success.append(keywords_good)

    # too many meta description
    if number_meta_description > 1:

        too_much_meta.found = number_meta_description
        too_much_meta.searched_in = meta_description
        site.warnings.append(too_much_meta)

    # perfect
    else:
        meta_description_only_one.searched_in = meta_description
        site.success.append(meta_description_only_one)

    # no meta description
    if not found_meta_description:
        site.problems.append(no_meta_description)

    # perfect
    else:
        meta_description_present.searched_in = meta_description
        site.success.append(meta_description_present)



# Third party
from django.utils.translation import gettext as _, pgettext

# Local application / specific library imports
from ..checks import custom_list


def importance():
    """Scripts with higher importance will be executed in first.

    Returns:
        int -- Importance of the script.
    """
    return 1


def run(site):
    """Checks that only one title tag is present, that its content is within a certain range, and that it contains at least one keyword.

    Arguments:
        site {Site} -- Structure containing a good amount of resources from the targeted webpage.
    """

    no_title = custom_list.CustomList(
        name=_("No meta title tag"),
        settings=pgettext("feminin", "one"),
        found=_("none"),
        description=_(
            "Titles tags are ones of the most important things to add to your pages, sinces they are the main text displayed on result search pages."
        ),
    )

    too_much = custom_list.CustomList(
        name=_("Too much meta title tags"),
        settings=pgettext("feminin", "only one"),
        description=_(
            "Only the first meta title tag will be displayed on the tab space on your browser, and only one meta title tag will be displayed on the search results pages."
        ),
    )

    short_title = custom_list.CustomList(
        name=_("Meta title tag is too short"),
        settings=_("more than {}").format(
            site.settings.SEO_SETTINGS["meta_title_length"][0]
        ),
        description=_(
            "Meta titles tags need to describe the content of the page, and need to contain at least a few words."
        ),
    )

    title_okay = custom_list.CustomList(
        name=_("Meta title tag have a good length"),
        settings=_("more than {}").format(
            site.settings.SEO_SETTINGS["meta_title_length"][0]
        ),
        description=_("Meta titles tags need to describe the content of the page."),
    )

    long_title = custom_list.CustomList(
        name=_("Meta title tag is too long"),
        settings=_("less than {}").format(
            site.settings.SEO_SETTINGS["meta_title_length"][1]
        ),
        description=_(
            "Only the first ~55-60 chars are displayed on modern search engines results. Writing a longer meta title is not really required and can lead to make the user miss informations."
        ),
    )

    no_keyword = custom_list.CustomList(
        name=_("Meta title tag do not contain any keyword"),
        settings=_("at least one"),
        found=_("none"),
        description=_(
            "Meta titles tags need to contain at least one keyword, since they are one of the most important content of the page for search engines."
        ),
    )

    keyword = custom_list.CustomList(
        name=_("Keywords found in meta title tag"),
        settings=_("at least one"),
        description=no_keyword.description,
    )

    # title presence
    titles = site.soup.find_all("title")
    if len(titles) < 1 or titles[0] is None or titles == "None":
        site.problems.append(no_title)
        return

    # multiple titles
    elif titles and len(titles) > 1:
        too_much.found = len(titles)
        too_much.searched_in = [t.string for t in titles]
        site.problems.append(too_much)

    # title length too short
    if len(titles[0].string) < site.settings.SEO_SETTINGS["meta_title_length"][0]:
        short_title.found = len(titles[0].string)
        short_title.searched_in = [titles[0].string]
        site.problems.append(short_title)

    # title length too long
    elif len(titles[0].string) > site.settings.SEO_SETTINGS["meta_title_length"][1]:
        long_title.found = len(titles[0].string)
        long_title.searched_in = [titles[0].string]
        site.warnings.append(long_title)
    else:
        title_okay.found = len(titles[0].string)
        title_okay.searched_in = [titles[0].string]
        site.success.append(title_okay)

    keyword.found = ""
    keyword_found = False
    title_readable = titles[0].string.lower()

    for kw in site.keywords:
        if kw.lower() in title_readable:
            if keyword_found:
                keyword.found += ", "
            keyword_found = True
            keyword.found += kw

    # title do not contain any keyword
    if keyword.found == "":
        no_keyword.searched_in = [titles[0].string]
        site.problems.append(no_keyword)
    else:
        keyword.searched_in = [titles[0].string]
        site.success.append(keyword)



# Standard Library
import json

# Third party
from bs4 import BeautifulSoup
from django.contrib.sites.models import Site
from django.utils.translation import gettext as _, ngettext
from django.views import generic
import requests

# Local application / specific library imports
from .checks import site
from .checks_list import launch_checks
from .conf import settings


class IndexView(generic.base.TemplateView):
    template_name = "default.html"

    def get_context_data(self, *args, **kwargs):
        context = super(generic.base.TemplateView, self).get_context_data(
            *args, **kwargs
        )

        if settings.DJANGO_CHECK_SEO_FORCE_HTTP:
            protocol = "http://"
        else:
            protocol = "https://"

        # get content of the page
        if "http" not in self.request.GET.get(
            "page", None
        ):  # url like "/fr/article-du-site/"
            full_url = (
                protocol
                + Site.objects.get_current().domain
                + self.request.GET.get("page", None)
            )
        else:  # url like "http://mydomain.ext/en/my-page-name/"
            full_url = self.request.GET.get("page", None)

        # use credentials if provided (pass through .htaccess auth)
        if (
            settings.DJANGO_CHECK_SEO_AUTH
            and settings.DJANGO_CHECK_SEO_AUTH["user"] is not None
            and settings.DJANGO_CHECK_SEO_AUTH["pass"] is not None
        ):
            r = requests.get(
                full_url,
                auth=(
                    settings.DJANGO_CHECK_SEO_AUTH["user"],
                    settings.DJANGO_CHECK_SEO_AUTH["pass"],
                ),
                headers={"Cache-Control": "no-cache"},
            )
        else:
            r = requests.get(full_url, headers={"Cache-Control": "no-cache"})

        soup = BeautifulSoup(r.text, features="lxml")

        # populate class with data
        page_stats = site.Site(soup, full_url)

        # magic happens here!
        launch_checks.launch_checks(page_stats)

        (context["problems"], context["warnings"], context["success"]) = (
            page_stats.problems,
            page_stats.warnings,
            page_stats.success,
        )

        context["settings"] = json.dumps(settings.SEO_SETTINGS, indent=4)
        context["html"] = page_stats.content
        context["text"] = page_stats.content_text
        context["keywords"] = page_stats.keywords

        nb_problems = len(context["problems"])
        nb_warnings = len(context["warnings"])

        # define some fancy-looking text here because it is wayyy to dirty with all the {% trans %} in template
        if nb_problems == 0 and nb_warnings == 0:
            context["nb_problems_warnings"] = _(
                '<strong class="green-bg">No problem</strong> was found on the page!'
            )
        else:
            if nb_problems == 0:
                context["nb_problems_warnings"] = _(
                    '<strong class="green-bg">No problem</strong> found, and '
                )
            else:
                context["nb_problems_warnings"] = '<strong class="red-bg">'
                context["nb_problems_warnings"] += ngettext(
                    "{nb_problems} problem</strong> found, and ",
                    "{nb_problems} problems</strong> found, and ",
                    nb_problems,
                ).format(nb_problems=nb_problems)

            if nb_warnings == 0:
                context["nb_problems_warnings"] += _(
                    '<strong class="green-bg">no warning</strong> raised'
                )
            else:
                context["nb_problems_warnings"] += '<strong class="yellow-bg">'
                context["nb_problems_warnings"] += ngettext(
                    "{nb_warnings} warning</strong> raised",
                    "{nb_warnings} warnings</strong> raised",
                    nb_warnings,
                ).format(nb_warnings=nb_warnings)

        return context



# Third party
# Standard Library
import sys

from django.conf import settings
from django.conf.global_settings import LANGUAGES
from django.utils.text import slugify
from django.utils.translation import gettext as _, pgettext

# Local application / specific library imports
from ..checks import custom_list


# hacky trick to add python2 compatibility to a python3 project after python2 eol
if sys.version_info.major == 2:
    from urlparse import urlparse
else:
    from urllib.parse import urlparse


def importance():
    """Scripts with higher importance will be executed in first.

    Returns:
        int -- Importance of the script.
    """
    return 1


def run(site):
    """Check presence of keywords in url.

    Arguments:
        site {Site} -- Structure containing a good amount of resources from the targeted webpage.
    """

    no_keyword = custom_list.CustomList(
        name=_("No keyword in URL"),
        settings=pgettext("masculin", "at least one"),
        found=pgettext("masculin", "none"),
        description=_(
            'Keywords in URL will help your users understand the organisation of your website, and are a small ranking factor for Google. On the other hand, Bing guidelines advises to "<i>keep [your URL] clean and keyword rich when possible</i>".'
        ),
    )

    enough_keyword = custom_list.CustomList(
        name=_("Keywords found in URL"),
        settings=pgettext("masculin", "at least one"),
        found="",
        description=no_keyword.description,
    )

    # root url may contain str like "/fr/" or "/en/" if i18n is activated
    url_path = urlparse(site.full_url, "/").path

    # list of languages from django LANGUAGES list: ['fr', 'en', 'br', 'ia', ...]
    languages_list = [i[0] for i in LANGUAGES]

    # do not check keywords in url for root URL
    if (
        (settings.USE_I18N and url_path.replace("/", "") in languages_list)
        or url_path == "/"
        or not url_path
    ):
        return

    keyword_found = False

    for keyword in site.keywords:
        if slugify(keyword) in site.full_url:
            if keyword_found:
                enough_keyword.found += ", "
            keyword_found = True
            enough_keyword.found += keyword

    if keyword_found:
        enough_keyword.searched_in = [site.full_url]
        site.success.append(enough_keyword)
    else:
        no_keyword.searched_in = [site.full_url]
        site.problems.append(no_keyword)



# Standard Library
import re

# Third party
from django.utils.translation import gettext as _, pgettext

# Local application / specific library imports
from ..checks import custom_list


def importance():
    """Scripts with higher importance will be executed in first.

    Returns:
        int -- Importance of the script.
    """
    return 1


def run(site):
    """Checks that h2 tags exist, and that at least one h2 contains at least one keyword.

    Arguments:
        site {Site} -- Structure containing a good amount of resources from the targeted webpage.
    """

    no_h2 = custom_list.CustomList(
        name=_("No h2 tag"),
        settings=_("at least one"),
        found=pgettext("masculin", "none"),
        description=_(
            "H2 tags are useful because they are explored by search engines and can help them understand the subject of your page."
        ),
    )

    enough_h2 = custom_list.CustomList(
        name=_("H2 tags were found"),
        settings=pgettext("feminin", "at least one"),
        description=no_h2.description,
    )

    no_keywords = custom_list.CustomList(
        name=_("No keyword in h2 tags"),
        settings=pgettext("masculin", "at least one"),
        found=pgettext("masculin", "none"),
        description=_(
            "Google uses h2 tags to better understand the subjects of your page."
        ),
    )

    enough_keywords = custom_list.CustomList(
        name=_("Keyword found in h2 tags"),
        settings=pgettext("masculin", "at least one"),
        found=pgettext("masculin", "none"),
        description=no_keywords.description,
    )

    h2 = site.soup.find_all("h2")
    if not h2:
        site.warnings.append(no_h2)
    else:
        enough_h2.found = len(h2)
        enough_h2.searched_in = [t.text for t in h2]
        site.success.append(enough_h2)

        occurence = []
        # check if each keyword
        for keyword in site.keywords:
            # is present at least
            for single_h2 in h2:
                occurence.append(
                    sum(
                        1
                        for _ in re.finditer(
                            r"\b%s\b" % re.escape(keyword.lower()),
                            single_h2.text.lower(),
                        )
                    )
                )
        # if no keyword is found in h2
        if not any(i > 0 for i in occurence):
            no_keywords.searched_in = [t.text for t in h2]
            site.warnings.append(no_keywords)
        else:
            enough_keywords.searched_in = [t.text for t in h2]
            enough_keywords.found = max(i for i in occurence)
            site.success.append(enough_keywords)



# Standard Library
import importlib
import sys

# Local application / specific library imports
from . import *  # noqa: F403,F401


# hacky thing aiming to add python2 compatibility after eol of python2
try:
    ModuleNotFoundError
except NameError:
    ModuleNotFoundError = ImportError

try:
    from checks_list import *  # noqa: F403,F401

    nomodules = False
except ModuleNotFoundError:
    nomodules = True


def launch_checks(site):
    """All the checks are performed here. Called in get_context_data().
    All functions should do its test(s), then add a dict in site.problems or site.warnings.

    Arguments:
        site {Site} -- A set of useful vars that can be used by the functions (including problems & warnings, two lists of dict).
    """

    modules_order = []

    # hacky trick to add python2 compatibility to a python3 project after python2 eol
    python_2_compatibility_array = [
        "django_check_seo.checks_list.launch_checks",
        "django_check_seo.checks_list.glob",
        "django_check_seo.checks_list.re",
        "django_check_seo.checks_list.bs4",
        "django_check_seo.checks_list.sys",
        "django_check_seo.checks_list.os",
        "django_check_seo.checks_list.importlib",
        "django_check_seo.checks_list.urlparse",
        "django_check_seo.checks_list.django",
    ]

    # only get modules in ...checks.*
    for module_name in sys.modules:
        if (
            "django_check_seo.checks_list." in module_name
            and module_name not in python_2_compatibility_array
        ) or (module_name.startswith("checks_list.")):
            module = importlib.import_module(module_name)
            get_module_order = getattr(module, "importance")

            # get the importance
            modules_order.append([module, get_module_order()])

    # execute modules with higher importance first from sorted list
    for module in sorted(modules_order, key=lambda x: x[1], reverse=True):
        getattr(module[0], "run")(site)



# Third party
from django.utils.translation import gettext as _, pgettext

# Local application / specific library imports
from ..checks import custom_list


def importance():
    """Scripts with higher importance will be executed in first.

    Returns:
        int -- Importance of the script.
    """
    return 5


def run(site):
    """Checks that meta tag exists and contain at least one keyword.
    Populate site.keywords list with keywords found.

    Arguments:
        site {Site} -- Structure containing a good amount of resources from the targeted webpage.
    """

    no_keywords = custom_list.CustomList(
        name=_("No keywords in meta keywords field"),
        settings=pgettext("masculin", "at least one"),
        found=pgettext("masculin", "none"),
        description=_(
            "Django-check-seo uses the keywords in the meta keywords field to check all other tests related to the keywords. A series of problems and warnings are related to keywords, and will therefore systematically be activated if the keywords are not filled in."
        ),
    )

    keywords_found = custom_list.CustomList(
        name=_("Keywords found in meta keywords field"),
        settings=pgettext("masculin", "at least one"),
        found="",
        description=no_keywords.description,
    )

    meta = site.soup.find_all("meta")
    for tag in meta:
        if (
            "name" in tag.attrs
            and tag.attrs["name"] == "keywords"
            and "content" in tag.attrs
            and tag.attrs["content"] != ""
        ):
            # get keywords for next checks
            site.keywords = tag.attrs["content"].split(
                ",  "
            )  # may be dangerous to hard code the case where keywords are separated with a comma and two spaces

            keywords_found.found = len(site.keywords)
            keywords_found.searched_in = site.keywords
            site.success.append(keywords_found)

            return

    site.problems.append(no_keywords)



# Standard Library
import re

# Local application / specific library imports
from ..conf import settings


class Site:
    """Structure containing a good amount of resources from the targeted webpage:
    - the settings
    - the soup (from beautifulsoup)
    - the content (all html except header & menu)
    - the full url
    - the keywords
    - the problems & warnings
    """

    def __init__(self, soup, full_url):
        """Populate some vars.

        Arguments:
            soup {bs4.element} -- beautiful soup content (html)
            full_url {str} -- full url
        """
        self.settings = settings

        self.soup = soup

        # remove footers
        if self.soup.find("footer"):
            self.soup.find("footer").extract()

        # remove menus
        if self.soup.find("ul", {"class": "nav"}):
            self.soup.find("ul", {"class": "nav"}).extract()
        elif self.soup.find("ul", {"class": "navbar"}):
            self.soup.find("ul", {"class": "navbar"}).extract()
        elif self.soup.find("nav"):
            self.soup.find("nav").extract()

        # Search all content blocks of the page
        self.content = self.soup.select(".container")

        # If no content block is found, then select all the content
        if self.content is None or self.content == []:
            self.content = self.soup.find_all("body")

        # get content without doublewords thx to custom separator ("<h1>Title</h1><br /><p>Content</p>" -> TitleContent)
        self.content_text = ""
        for c in self.content:
            self.content_text += c.get_text(separator=" ")

        # strip multiple carriage return (with optional space) to only one
        self.content_text = re.sub(r"(\n( ?))+", "\n", self.content_text)
        # strip multiples spaces (>3) to only 2 (for title readability)
        self.content_text = re.sub(r"   +", "  ", self.content_text)

        self.full_url = full_url
        self.keywords = []
        self.problems = []
        self.warnings = []
        self.success = []



# Third party
# hacky trick to add python2 compatibility to a python3 project after python2 eol
import django
from django.contrib.admin.views.decorators import staff_member_required

# Local application / specific library imports
from . import views


version = django.get_version()

if version.startswith("2"):
    from django.urls import path

    urlpatterns = [
        path("", staff_member_required(views.IndexView.as_view()), name="Index")
    ]

else:
    from django.conf.urls import url

    urlpatterns = [
        url("^.*$", staff_member_required(views.IndexView.as_view()), name="Index"),
    ]



# Third party
from django.utils.translation import gettext as _, pgettext

# Local application / specific library imports
from ..checks import custom_list


def importance():
    """Scripts with higher importance will be executed in first.

    Returns:
        int -- Importance of the script.
    """
    return 1


def run(site):
    """Checks that at least one keyword is included in the first paragraph.

    Arguments:
        site {Site} -- Structure containing a good amount of resources from the targeted webpage.
    """

    no_keywords = custom_list.CustomList(
        name=_("No keyword in first paragraph"),
        settings=_("before {settings} words").format(
            settings=site.settings.SEO_SETTINGS["keywords_in_first_words"]
        ),
        found=pgettext("masculin", "none"),
        description=_(
            "The reader will be relieved to find one of his keywords in the first paragraph of your page, and the same logic applies to Google, which will consider the content more relevant."
        ),
    )

    found = False
    first_N_words = site.content_text.lower().split()[
        : site.settings.SEO_SETTINGS["keywords_in_first_words"]
    ]
    # check text and not list of words in order that keywords like "this is a keyword" are found in text "words this is a keyword words"
    first_N_words = " ".join(first_N_words)

    nb = 0
    kw = []
    for keyword in site.keywords:
        if keyword.lower() in first_N_words:
            found = True
            kw.append(keyword)
        nb += 1

    no_keywords.searched_in = [first_N_words]

    # no keyword was found in first paragraph
    if not found:
        site.problems.append(no_keywords)

    else:
        no_keywords.name = _("Keywords found in first paragraph")
        no_keywords.found = ", ".join(kw)
        site.success.append(no_keywords)



# Third party
import bs4
from django.utils.translation import gettext as _

# Local application / specific library imports
from ..checks import custom_list


def importance():
    """Scripts with higher importance will be executed in first.

    Returns:
        int -- Importance of the script.
    """
    return 1


def run(site):
    """Checks that each image has a filled alt tag.

    Arguments:
        site {Site} -- Structure containing a good amount of resources from the targeted webpage.
    """

    lack_alt = custom_list.CustomList(
        name=_("Img lack alt tag"),
        settings=_("all images"),
        found="",
        description=_(
            'Your images should always have an alt tag, because it improves accessibility for visually impaired people. The name of the file is important too, because it helps Google understand what your image is about. For example, you could rename a file named "IMG0001.jpg" to "tree_with_a_bird.jpg".'
        ),
    )

    enough_alt = custom_list.CustomList(
        name=_("Img have alt tag"),
        settings=_("all images"),
        found="",
        description=lack_alt.description,
    )

    images = bs4.element.ResultSet(None)

    for c in site.content:
        images += c.find_all("img")

    problem = 0
    imgs = []
    img_str = _("image")

    for image in images:
        if (
            "alt" not in image.attrs
            or image.attrs["alt"] == "None"
            or image.attrs["alt"] == ""
        ):
            problem += 1

            # bold without alt tag content
            if image.attrs["src"] != "":
                imgs.append(
                    '<b><u><a target="_blank" href="'
                    + image.attrs["src"]
                    + '">'
                    + img_str
                    + "</a></u></b>"
                )
            # bold without alt tag content & without src content (dead img ?)
            else:
                imgs.append("<b><u>unknown image</u></b>")

        # normal with alt tag content
        else:
            imgs.append(
                '<a target="_blank" href="'
                + image.attrs["src"]
                + '">'
                + img_str
                + "</a> ("
                + image.attrs["alt"]
                + ")"
            )

    if problem > 0:
        lack_alt.found = problem
        lack_alt.searched_in = imgs
        site.problems.append(lack_alt)
    else:
        if len(images) > 0:
            enough_alt.found = len(images)
            enough_alt.searched_in = imgs
            site.success.append(enough_alt)



# Third party
from django.utils.translation import gettext as _

# Local application / specific library imports
from ..checks import custom_list


def importance():
    """Scripts with higher importance will be executed in first.

    Returns:
        int -- Importance of the script.
    """
    return 1


def run(site):
    """Check the length of the url and the maximum depth.

    Arguments:
        site {Site} -- Structure containing a good amount of resources from the targeted webpage.
    """

    deep_url = custom_list.CustomList(
        name=_("Too many levels in path"),
        settings=_("less than {}").format(site.settings.SEO_SETTINGS["max_link_depth"]),
        description=_(
            "Google recommand to organize your content by adding depth in your url, but advises against putting too much depth."
        ),
    )

    long_url = custom_list.CustomList(
        name=_("URL is too long"),
        settings=_("less than {}").format(site.settings.SEO_SETTINGS["max_url_length"]),
        description=_("Shorter URLs tend to rank better than long URLs."),
    )

    # check url depth
    # do not count first and last slashes (after domain name and at the end of the url), nor // in the "http://"
    url_without_two_points_slash_slash = site.full_url.replace("://", "")
    number_of_slashes = url_without_two_points_slash_slash[:-1].count("/")

    deep_url.found = number_of_slashes
    deep_url.searched_in = [site.full_url]

    if number_of_slashes > site.settings.SEO_SETTINGS["max_link_depth"]:
        site.problems.append(deep_url)
    else:
        deep_url.name = _("Right amount of level in path")
        site.success.append(deep_url)

    # check url length
    url_without_protocol = site.full_url.replace("http://", "").replace("https://", "")
    long_url.found = len(url_without_protocol)
    long_url.searched_in = [url_without_protocol]

    if len(url_without_protocol) > site.settings.SEO_SETTINGS["max_url_length"]:
        site.warnings.append(long_url)
    else:
        long_url.name = _("URL length is great")
        site.success.append(long_url)



class CustomList:
    def __init__(self, *args, **kwargs):
        self.name = kwargs.get("name", None)
        self.settings = kwargs.get("settings", None)
        self.found = kwargs.get("found", None)
        self.searched_in = kwargs.get("searched_in", [])
        self.description = kwargs.get("description", None)



# Standard Library
import glob
from os.path import basename, dirname, isfile, join


# list files
modules = glob.glob(join(dirname(__file__), "*.py"))

__all__ = []

# add them to __all__ so they can be imported
for module in modules:
    if (
        isfile(module)
        and not module.endswith("__init__.py")
        and not module.endswith("launch_checks.py")
    ):
        __all__.append(basename(module)[:-3])



# Third party
import bs4
from django.contrib.sites.models import Site
from django.utils.translation import gettext as _

# Local application / specific library imports
from ..checks import custom_list


def importance():
    """Scripts with higher importance will be executed in first.

    Returns:
        int -- Importance of the script.
    """
    return 1


def run(site):
    """Counts the number of internal and external links in the extracted content.

    Arguments:
        site {Site} -- Structure containing a good amount of resources from the targeted webpage.
    """

    not_enough_internal = custom_list.CustomList(
        name=_("Not enough internal links"),
        settings=_("at least {}").format(site.settings.SEO_SETTINGS["internal_links"]),
        description=_(
            "Internal links are useful because they can give the structure of your website to search engines, so they can create a hierarchy of your pages."
        ),
    )

    enough_internal = custom_list.CustomList(
        name=_("Internal links were found"),
        settings=_("at least {}").format(site.settings.SEO_SETTINGS["internal_links"]),
        description=not_enough_internal.description,
    )

    not_enough_external = custom_list.CustomList(
        name=_("Not enough external links"),
        settings=_("at least {}").format(site.settings.SEO_SETTINGS["external_links"]),
        description=_(
            "External links help your users to check your topic and can save them from having to do additional research."
        ),
    )

    enough_external = custom_list.CustomList(
        name=_("External links were found"),
        settings=_("at least {}").format(site.settings.SEO_SETTINGS["external_links"]),
        description=not_enough_external.description,
    )

    links = bs4.element.ResultSet(None)

    # only get links with href
    for c in site.content:
        links += c.find_all("a", href=True)

    internal_links = 0
    internal_links_list = []
    external_links_list = []
    external_links = 0

    for link in links:
        # internal links = absolute links that contains domain name or relative links
        if Site.objects.get_current().domain in link["href"] or not link[
            "href"
        ].startswith("http"):
            internal_links += 1
            internal_links_list.append(
                '<a href="' + link["href"] + '">' + link.text + "</a>"
            )
        else:
            external_links += 1
            external_links_list.append(
                '<a href="' + link["href"] + '">' + link.text + "</a>"
            )

    # not enough internal links
    if internal_links < site.settings.SEO_SETTINGS["internal_links"]:
        not_enough_internal.found = internal_links
        not_enough_internal.searched_in = internal_links_list
        site.warnings.append(not_enough_internal)
    else:
        enough_internal.found = internal_links
        enough_internal.searched_in = internal_links_list
        site.success.append(enough_internal)

    # not enough external links
    if external_links < site.settings.SEO_SETTINGS["external_links"]:
        not_enough_external.found = external_links
        not_enough_external.searched_in = external_links_list
        site.warnings.append(not_enough_external)
    else:
        enough_external.found = external_links
        enough_external.searched_in = external_links_list
        site.success.append(enough_external)



# see https://github.com/kapt-labs/django-check-seo/wiki/Toolbar-shortcut#cms_toolbarspy

# Third party
from cms.toolbar_base import CMSToolbar
from cms.toolbar_pool import toolbar_pool
from django.utils.translation import ugettext_lazy as _


class DjangoSeoToolbar(CMSToolbar):
    def populate(self):

        self.toolbar.add_sideframe_item(
            _("Check SEO"),  # text
            "/django-check-seo/?page="
            + str(self.request.path),  # url (+ current page passed as a GET parameter)
        )


# register the toolbar
toolbar_pool.register(DjangoSeoToolbar)



# Third party
from django.conf import settings


# define basic SEO settings
SEO_SETTINGS = {
    "content_words_number": [300, 600],
    "internal_links": 1,
    "external_links": 1,
    "meta_title_length": [30, 60],
    "meta_description_length": [50, 160],
    "keywords_in_first_words": 50,
    "max_link_depth": 3,
    "max_url_length": 70,
}
# update SEO settings with values from projectname/settings.py
SEO_SETTINGS.update(getattr(settings, "DJANGO_CHECK_SEO_SETTINGS", {}))


# define auth data (for .htaccess files)
DJANGO_CHECK_SEO_AUTH = {}
# update auth data with values from projectname/settings.py
DJANGO_CHECK_SEO_AUTH.update(getattr(settings, "DJANGO_CHECK_SEO_AUTH", {}))

# define http(s) settings (default = use https)
DJANGO_CHECK_SEO_FORCE_HTTP = False
# update http(s) settings with value from projectname/settings.py
DJANGO_CHECK_SEO_FORCE_HTTP = getattr(settings, "DJANGO_CHECK_SEO_FORCE_HTTP", False)
